%plot_normalized_histograms
% Plots a normalized histogram from an array of N samples of n elements of random
% numbers sampled from a particular continuous probability distribution.
% This is overlaid with the form of the probability distribution.
% In addition, the distribution of sample means is plotted as a normalized
% histogram, and overlaid with a Normal distribution. If n is large enough,
% the Central Limit Theorem states the Normal distribution should be what
% the normalized histogram tends towards, regardless of the population
% distribution!
%
% Syntax: plot_normalized_histograms(x,xx,px,mu,sigma,num_edges,name,title_str)
%
% x          n x N array of random numbers. Sample size is n. N is number of samples.
% xx         Array of x values for probability density function
% px         Corresponding probability density function to xx
% mu         Population mean
% sigma      Population standard deviation
% num_edges  Number of bin edges for normalized histogram
% name       File name for PNG output
% title_str  Title string for graphs
% fsize      Fontsize for graphs
%
% LAST UPDATED by Dr Andrew French September 2017

function plot_normalized_histograms(x,xx,px,mu,sigma,num_edges,name,title_str,fsize)

%Determine sample means (unbiased estimator of population mean = 1)
xbar = mean(x,1);

%Determin number per sample and total number of samples
[n,N] = size(x);

%Determine unbiased estimate of population standard deviation ( = sqrt(3/5) )
%from sample standard deviations
s = sqrt( n/(n-1) )*std(x);

%Determine sqrt of mean of variances
sigma_est = sqrt( mean(s.^2) );

%Determine normalized histogram of data in all samples
xmin = min(xx);
xmax = max(xx);
edges = linspace( xmin,xmax, num_edges );
dx = ( xmax - xmin)/num_edges;
x = reshape(x,[1,n*N]);
hx = histc(x,edges )/(n*N*dx);

%Check that sum of hx*dx values equals unity
norm_histogram_check = sum(hx*dx);

%Determine p(xbar) Normal distribution of sample means
sgma = sigma/sqrt(n);
xxbar = linspace( xmin,xmax,1000);
p_xbar = ( 1/sqrt( 2*pi*sgma^2))*exp( -((xxbar - mu).^2)/(2*sgma^2) );

%Determine normalized histogram of sample means, using 100 bin edges
h_xbar = histc(xbar, edges)/(N*dx);

%

%Plot normalized histogram of data samples and overlay p(x)
b = bar( edges, hx, 'histc' );
set(b,'facecolor',[0,1,0]);
hold on;
plot( xx, px,'r-');
xlabel('x','fontsize',fsize);
ylabel('Probability density','fontsize',fsize);
title( title_str,'fontsize',fsize);
grid on;
box on;
set(gca,'fontsize',fsize);
xlim([xmin,xmax])
print(gcf,[name,'.png'],'-dpng','-r300');
clf;

%

%Plot normalized histogram and overlay normal distribution to illustrate
%Central Limit Theorem
b = bar( edges, h_xbar, 'histc' );
set(b,'facecolor',[0,1,0]);
hold on;
plot( xxbar, p_xbar,'r-');
xlabel('Sample mean xbar','fontsize',fsize);
ylabel('Probability density','fontsize',fsize);
title({['Normalized histogram of N sample means. n = ',...
    num2str(n),', N = ',num2str(N)],...
    ['\sigma estimate = ',num2str(sigma_est),', \sigma = ',...
    num2str(sigma),', \mu estimate = ',num2str(mean(xbar)),', \mu = ',num2str(mu)]},...
    'fontsize',fsize);
grid on;
box on;
set(gca,'fontsize',fsize);
xlim([xmin,xmax])
print(gcf,[name,' sample means.png'],'-dpng','-r300');
close(gcf);

%

%Save data to a .mat file
save(name,'x','xx','px','mu','sigma','num_edges',...
    'name','title_str','fsize','xbar','s','sigma_est')

%End of code




