%weibull
% Function which demonstrates the generation of random numbers from an
% Weibull distribution. Samples are taken from this distribution, and the
% distribution of sample means is determined via a normalized histogram.
% From this the Central Limit Theorem is illustrated. i.e. the distribution
% of sample means tends to a normal distribution if the number of elements
% per sample is large enough.
%
% LAST UPDATED by Dr Andrew French September 2017

function weibull

%% INPUTS %%

%Define weibull parameters (these need to both be positive)
k = 5;
lambda = 3;

%Define number of samples
N = 1e4;

%Define number of elements per sample
n = 42;

%Define number of elements in probability density
npx = 1000;

%Number of histogram bin edges over range [xmin,xmax]
num_edges = 150;

%Calculate Weibull distribution mean and standard deviation
mu = lambda*gamma( 1 + 1/k );
sigma = sqrt( (lambda^2)*gamma( 1 + 2/k ) - mu^2 );

%Range of x values for probability density function
xmin = 0;
xmax = mu + 4*sigma;

%Define title string for this distribution
title_str = {['Normalized histogram of N=',...
    num2str(N),' samples of n=',num2str(n)],...
    ['Weibull distribution \mu=',num2str(mu),', \sigma=',num2str(sigma),', k=',num2str(k),', \lambda=',num2str(lambda)]};

%Define filename for PNG output
name = 'weibull';

%Fontsize for graphs
fsize = 18;

%

%Define probability density function
xx = linspace(xmin,xmax,1000);
px = (k/lambda)*( (xx/lambda).^(k-1) ).*exp(-( (xx/lambda).^k ) );

%Generate an n by N array of uniformly distributed random numbers in the
%range [0,1]
z = rand(n,N);

%Use the inverse function of the probability P(x) to compute x values
x = lambda*nthroot( -log( 1-z) ,k );

%Plot normalized histogram of x data, and sample means
plot_normalized_histograms(x,xx,px,mu,sigma,num_edges,name,title_str,fsize)

%End of code




