%Generate random numbers, plot normalized histogram and overlay with
%probability distribution.
%
% Syntax: generate_numbers_and_histogram(N,P,xx,title_str,name,fsize)
%
% N           Number of numbers to generate
% P           Vector of probabilities, corresponding to integers 0...length(P)-1
% title_str   Title string for normalized histogram plot
% name        Filename for PNG output e.g. 'binomial'
% fsize       Fontsize for graph

% LAST UPDATED by Andy French September 2017

function generate_numbers_and_histogram(N,P,title_str,name,fsize)

%Determine array of cumulative probabilities
CP = cumsum(P);
if CP(end)~=1
    CP = [CP,1];
end

%Generate N uniformly distributed random numbers in range [0,1]
z = rand(1,N);

%Define an array of cumulative probability, one element shifted.
%This is used to efficiently determine the 'bin' corresponding to z
CP0 = [0,CP(1:end-1)];

%For each number, determine the 'bin' number of a division of the interval
%[0,1] by the cumulative probabilities. If the first bin is zero, then
%these numbers should be distributed according to the probabililty
%distribution used to generate the cumulative probabilities.
x = zeros(1,N);
for k=1:length(CP)
    test = ( z >= CP0(k) ) & ( z <= CP(k) );
    x( test ) = k - 1;
end

%Ignore any values from final bin, if CP is longer than P
if numel(CP)>numel(P)
    x( x==numel(P) ) = [];
end

%Determine a normalized histogram of x values
xx = 0 : length(P) - 1;
h = hist( x, xx )/N;

%Plot normalized histogram
b = bar(xx,h);
set(b,'facecolor','g');
xlabel('x','fontsize',fsize);
ylabel('Probability','fontsize',fsize);
title(title_str,'fontsize',fsize);
grid on;
box on;
set(gca,'fontsize',fsize);
xlim([0,max(xx)+1]);

%Overlay probability distribution
hold on
plot(xx,P,'r*');

%Print PNG of graph
print(gcf,[name,'.png'],'-dpng','-r300');
close(gcf);

%Save data
save(name,'h','xx','x','N','P','title_str','name','fsize');

%End of code